```wmd-options
language = es
title_i18n = errors_manager_title_type
title_text = type - ErrorsManager
```

<!-- [[wmd]] -->

### type

[[@ [String] ErrorsManager:type(!?Any value)]]

El método **type** es un método estático que retorna el tipado de una variable con respecto a si es:

* **[[!string]]**: Si es un texto.
* **[[!integer]]**: Si es un valor numérico entero.
* **[[!array]]**: Si es un Array, Vector, Lista, Tupla, etc. De valores numéricos enteros.
* **[[!unknown]]**: Si no es ninguno de los tipos anteriores.

> [!!] La librería sólo trabaja los tipados String, Integer y Array de enteros. Si es otro tipado, ésta lo tomará como desconocido y lo tratará como algo inesperado o un error, pero no hará saltar una Excepción.

```py
#!/usr/bin/env python
# -*- coding: utf-8 -*-

from Assets.ErrorsManager import ErrorsManager

errors_manager:ErrorsManager = ErrorsManager()

print(["Prueba 1", errors_manager.type("")])
print(["Prueba 2", errors_manager.type("Hola xD")])
print(["Prueba 3", errors_manager.type(None)])
print(["Prueba 4", errors_manager.type(5)])
print(["Prueba 5", errors_manager.type(-8)])
print(["Prueba 6", errors_manager.type(-104.56)])
print(["Prueba 7", errors_manager.type(6.007)])
print(["Prueba 8", errors_manager.type(.345)])
print(["Prueba 9", errors_manager.type(ErrorsManager)])
print(["Prueba 10", errors_manager.type(errors_manager)])
print(["Prueba 11", errors_manager.type(["Esto es un String"])])
print(["Prueba 12", errors_manager.type(['S', 't', 'r', 'i', 'n', 'g'])])
print(["Prueba 13", errors_manager.type(True)])
print(["Prueba 14", errors_manager.type(False)])
print(["Prueba 15", errors_manager.type((True,))])
print(["Prueba 16", errors_manager.type([2, 45, 18, 24])])

```

```js
"use strict";

/** @type {ErrorsManager} */
const errors_manager = new ErrorsManager();

console.log(["Prueba 1", errors_manager.type("")]);
console.log(["Prueba 2", errors_manager.type("Hola xD")]);
console.log(["Prueba 3", errors_manager.type(null)]);
console.log(["Prueba 4", errors_manager.type(5)]);
console.log(["Prueba 5", errors_manager.type(-8)]);
console.log(["Prueba 6", errors_manager.type(-104.56)]);
console.log(["Prueba 7", errors_manager.type(6.007)]);
console.log(["Prueba 8", errors_manager.type(.345)]);
console.log(["Prueba 9", errors_manager.type(ErrorsManager)]);
console.log(["Prueba 10", errors_manager.type(errors_manager)]);
console.log(["Prueba 11", errors_manager.type(["Esto es un String"])]);
console.log(["Prueba 12", errors_manager.type(['S', 't', 'r', 'i', 'n', 'g'])]);
console.log(["Prueba 13", errors_manager.type(true)]);
console.log(["Prueba 14", errors_manager.type(false)]);
console.log(["Prueba 15", errors_manager.type([true])]);
console.log(["Prueba 16", errors_manager.type([2, 45, 18, 24])]);

```

En los ejemplos anteriores sacaríamos los siguientes resultados:

[|
|= Prueba   | Python         | JavaScript
|  1        | [[!string]]    | [[!string]]
|  2        | [[!string]]    | [[!string]]
|  3        | [[!unknown]]   | [[!unknown]]
|  4        | [[!integer]]   | [[!integer]]
|  5        | [[!integer]]   | [[!integer]]
|  6        | [[!unknown]]   | [[!unknown]]
|  7        | [[!unknown]]   | [[!unknown]]
|  8        | [[!unknown]]   | [[!unknown]]
|  9        | [[!unknown]]   | [[!unknown]]
|  10       | [[!unknown]]   | [[!unknown]]
|  11       | [[!array]]     | [[!array]]
|  12       | [[!array]]     | [[!array]]
|  13       | [[!unknown]]   | [[!unknown]]
|  14       | [[!unknown]]   | [[!unknown]]
|  15       | [[!array]]     | [[!array]]
|  16       | [[!array]]     | [[!array]]
|]

<!-- [[wmd]] -->